/**
 * A Binding is an environment in which one new name is bound to some
 * value, while all other names retain the values they have in some
 * pre-existing outer environment, accessed via a nesting link.
 */
public class Binding implements Environment {

    private String name;
    private Object value;
    private Environment nestingLink;

    /**
     * Creates a binding with an ordinary value.
     * This is the kind of binding used for ML's "val".
     * @param name the name being bound
     * @param value the value the name is bound to
     * @param nestingLink the environment being augmented
     */
    public Binding(String name,
                   Object value,
                   Environment nestingLink){
        // The following condition traps a common bug:
        //  using an unevaluated expression in place of the value.
        if(value instanceof Expression){
            throw new Error("The 'value' in a Binding was an Expression.");
        }
        this.name = name;
        this.value = value;
        this.nestingLink = nestingLink;
    }

    /**
     * Creates a binding for a (possibly recursive) function.
     * This is the kind of binding used for ML's "fun".
     * @param functionName the name being bound
     * @param parameterName the function's parameter
     * @param functionBody the function's body
     * @param nestingLink the environment being augmented
     */
    public Binding(String functionName,
                   String parameterName,
                   Expression functionBody,
                   Environment nestingLink){
        name = functionName;
        value = new Closure(parameterName, functionBody, this);
        this.nestingLink = nestingLink;
    }

    public Object get(String name){
        if(name.equals(this.name)){
            return value;
        } else {
            return nestingLink.get(name);
        }
    }
}